from typing import TYPE_CHECKING

from sqlalchemy import Column, ForeignKey, Integer, String, Enum, DateTime
from sqlalchemy.orm import relationship
from sqlalchemy.sql import func
from app.db.base_class import Base


if TYPE_CHECKING:
  from .user import User  # noqa: F401
  from .enum import Progress

class Video(Base):
  id = Column(Integer, primary_key=True, index=True)
  title = Column(String(40), index=True, nullable=False)
  stored_filename = Column(String(30), unique=True, nullable=False)
  progress_state = Column(String(10), 
                    ForeignKey("progress.state", ondelete="RESTRICT", onupdate="CASCADE"),
                    default="waiting")
  created_datetime = Column(DateTime(timezone=True), default=func.now())
  length = Column(Integer)
  owner_id = Column(Integer, ForeignKey("user.id"))
  owner = relationship("User", back_populates="videos")